﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Unitronics.ComConfig.Lite;

namespace SD_Card_Explorer
{
    public partial class CommunicationView : UserControl
    {
        public CommunicationView()
        {
            InitializeComponent();
            comConfig1.SupportedChannels = (int)ComConfig.Channels.Serial | (int)ComConfig.Channels.Ethernet;
            comConfig1.OnDirtyBitValueChanged += new Unitronics.ComConfig.Lite.ComConfig.DirtyBitValueChanged(comConfig1_OnDirtyBitValueChanged);
            loadComSettingsFromFile();
        }

        private void CommunicationView_Resize(object sender, EventArgs e)
        {
            int left = (this.Width - this.panel1.Width) / 2;
            int top = (this.Height - this.panel1.Height) / 2;

            if (left < 0)
                left = 0;

            if (top < 8)
                top = 8;

            this.panel1.Left = left;
            this.panel1.Top = top;
        }

        private void loadComSettingsFromFile()
        {
            string path = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase.Replace(@"file:///", ""));
            string dirSeparator = Path.DirectorySeparatorChar.ToString();
            if (path.Substring(path.Length - 1) == dirSeparator)
                path = path.Substring(0, path.Length - 1);

            // Using default settings if file doesn't exist, and saving it to disk
            if (!File.Exists(path + dirSeparator + "ComSettings.xml"))
            {
                comConfig1.Save(path + dirSeparator + "ComSettings.xml");
            }
            else
            {
                comConfig1.Load(path + dirSeparator + "ComSettings.xml");
            }

        }

        private void btnApplyChanges_Click(object sender, EventArgs e)
        {
            string path = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase.Replace(@"file:///", ""));
            string dirSeparator = Path.DirectorySeparatorChar.ToString();
            if (path.Substring(path.Length - 1) == dirSeparator)
                path = path.Substring(0, path.Length - 1);

            comConfig1.Save(path + dirSeparator + "ComSettings.xml");
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            if (comConfig1.DirtyBit)
            {
                string path = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase.Replace(@"file:///", ""));
                string dirSeparator = Path.DirectorySeparatorChar.ToString();
                if (path.Substring(path.Length - 1) == dirSeparator)
                    path = path.Substring(0, path.Length - 1);

                comConfig1.Load(path + dirSeparator + "ComSettings.xml");
            }
        }

        void comConfig1_OnDirtyBitValueChanged(bool value)
        {
            btnApplyChanges.Enabled = value;
        }

    }
}
