﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Unitronics.ComConfig.Lite.Presenters;
using Unitronics.ComDriver;

namespace Unitronics.ComConfig.Lite
{
    internal partial class FavoritesView : UserControl
    {
        public event EventHandler OnFavoriteRowLoad;
        private Presenters.FavoritesPresenter m_Presenter;
        private int lastSelectedRowIndex = 0;

        public FavoritesView()
        {
            InitializeComponent();
            m_Presenter = new FavoritesPresenter();
            m_Presenter.OnFavoritesListUpdate += new EventHandler(m_Presenter_OnFavoritesListUpdate);
        }

        public PlcComConfig PlcComConfig
        {
            get
            {
                return m_Presenter.PlcComConfig;
            }
            set
            {
                m_Presenter.PlcComConfig = value;
            }
        }

        public List<ComConfig.Channels> SupportedChannelsList
        {
            set
            {
                m_Presenter.SupportedChannelsList = value;
            }
        }

        void m_Presenter_OnFavoritesListUpdate(object sender, EventArgs e)
        {
            List<PlcComConfig> filteredConnections = (List<PlcComConfig>)sender;
            int index = 0;
            DataTable dt = new DataTable();
            dt.Columns.AddRange(new DataColumn[] { 
                new DataColumn("PlcComConfig", typeof(PlcComConfig)),
                new DataColumn("Connection Type"),
                new DataColumn("Unit ID"),
                new DataColumn("Require PLC Name"),
                new DataColumn("PLC Name")});

            foreach (PlcComConfig plcComConfig in filteredConnections)
            {
                List<object> rowParams = new List<object>();
                rowParams.Add(filteredConnections[index]);
                if (plcComConfig.Channel is Serial)
                {
                    Serial serial = plcComConfig.Channel as Serial;
                    rowParams.Add("Serial - " + serial.PortName.ToString() + ", " + serial.BaudRate.ToString());
                }
                else if (plcComConfig.Channel is Ethernet)
                {
                    Ethernet ethernet = plcComConfig.Channel as Ethernet;
                    rowParams.Add("Ethernet - " + ethernet.RemoteIP + ":" + ethernet.RemotePort.ToString());
                }
                else if (plcComConfig.Channel is EthernetListener)
                {
                    EthernetListener listener = plcComConfig.Channel as EthernetListener;
                    rowParams.Add("Ethernet (Listen) - Port: " + listener.LocalPort.ToString());
                }
                else
                {
                    rowParams.Add("");
                }

                rowParams.Add(plcComConfig.UnitID);
                if (plcComConfig.RequirePlcName)
                {
                    rowParams.Add("Yes");
                }
                else
                {
                    rowParams.Add("No");
                }
                
                rowParams.Add(plcComConfig.PlcName);

                dt.Rows.Add(rowParams.ToArray());
                index++;
            }

            gridFavorites.SuspendLayout();
            gridFavorites.Columns.Clear();
            gridFavorites.DataSource = dt;

            gridFavorites.Columns[0].Visible = false;
            for (int i = 0; i < gridFavorites.Columns.Count; i++)
            {
                gridFavorites.Columns[i].SortMode = DataGridViewColumnSortMode.NotSortable;
            }

            //for (int i = 0; i< gridFavorites.Rows.Count; i++)
            //{
            //    gridFavorites.Rows[i].Tag = filteredConnections[i];
            //}

            if (gridFavorites.Rows.Count > lastSelectedRowIndex)
            {
                gridFavorites.Rows[lastSelectedRowIndex].Selected = true;
            }
            else if (gridFavorites.Rows.Count > 0)
            {
                gridFavorites.Rows[gridFavorites.Rows.Count - 1].Selected = true;
            }

            gridFavorites.ResumeLayout();
        }

        private void toolStrip1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            switch (e.ClickedItem.Name)
            {
                case "favAdd":
                    addConnectionToFavorites();
                    break;
                case "favDelete":
                    deleteConnectionFromFavorites();
                    break;
                case "favLoad":
                    loadConnectionFromFavorites();
                    break;
            }
        }

        private void addConnectionToFavorites()
        {
            m_Presenter.AddConnectionToFavorites();
        }

        private void deleteConnectionFromFavorites()
        {
            if (gridFavorites.SelectedRows.Count == 0)
                return;

            PlcComConfig plcComConfig = (PlcComConfig)gridFavorites.SelectedRows[0].Cells[0].Value;
            m_Presenter.DeleteConnectionFromFavorites(plcComConfig);
        }

        private void loadConnectionFromFavorites()
        {
            if (gridFavorites.SelectedRows.Count == 0)
                return;

            PlcComConfig plcComConfig = (PlcComConfig)gridFavorites.SelectedRows[0].Cells[0].Value;
            PlcComConfig replica = Globals.Utils.GetPlcComConfigReplica(plcComConfig);

            if (OnFavoriteRowLoad != null)
                OnFavoriteRowLoad(replica, null);
        }

        private void gridFavorites_RowEnter(object sender, DataGridViewCellEventArgs e)
        {
            if (gridFavorites.SelectedRows.Count == 0)
                return;

            try
            {
                lastSelectedRowIndex = gridFavorites.SelectedRows[0].Index;
            }
            catch
            {
                lastSelectedRowIndex = 0;
            }
        }

        private void gridFavorites_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex < 0)
                return;

            DataGridViewRow row = gridFavorites.Rows[e.RowIndex];
            PlcComConfig plcComConfig = (PlcComConfig)row.Cells[0].Value;
            PlcComConfig replica = Globals.Utils.GetPlcComConfigReplica(plcComConfig);

            FavoritesEdit favEdit = new FavoritesEdit();
            if (plcComConfig.Channel is Serial)
            {
                SerialView serialView = new SerialView(replica);
                favEdit.SetView(serialView);
                serialView.Dock = DockStyle.Fill;
                favEdit.ShowDialog();
                if (favEdit.OkClicked)
                {
                    row.Cells[0].Value = replica;
                    Globals.Utils.SetPlcComConfigFromReplica(plcComConfig, replica);
                    m_Presenter.Save();
                }
            }
            else if (plcComConfig.Channel is Ethernet)
            {
                EthernetView ethernetView = new EthernetView(replica);
                favEdit.SetView(ethernetView);
                ethernetView.Dock = DockStyle.Fill;
                favEdit.ShowDialog();
                if (favEdit.OkClicked)
                {
                    row.Cells[0].Value = replica;
                    Globals.Utils.SetPlcComConfigFromReplica(plcComConfig, replica);
                    m_Presenter.Save();
                }
            }
            else if (plcComConfig.Channel is EthernetListener)
            {
                ListenerView listenerView = new ListenerView(replica);
                favEdit.SetView(listenerView);
                listenerView.Dock = DockStyle.Fill;
                favEdit.ShowDialog();
                if (favEdit.OkClicked)
                {
                    row.Cells[0].Value = replica;
                    Globals.Utils.SetPlcComConfigFromReplica(plcComConfig, replica);
                    m_Presenter.Save();
                }
            }
        }
    }
}
