﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms ;

using Unitronics.ComDriver;

namespace Unitronics.ComConfig.Lite.Globals {

    internal static class Utils
    {
        internal static PlcComConfig GetPlcComConfigReplica(PlcComConfig plcConfig)
        {
            PlcComConfig replica = new PlcComConfig();
            Channel channel = plcConfig.Channel;
            if (channel is Serial)
            {
                Serial serial = new Serial();
                Serial originalSerial = (plcConfig.Channel as Serial);
                serial.PortName = originalSerial.PortName;
                serial.BaudRate = originalSerial.BaudRate;
                serial.DataBits = originalSerial.DataBits;
                serial.StopBits = originalSerial.StopBits;
                serial.Parity = originalSerial.Parity;
                serial.Retry = originalSerial.Retry;
                serial.TimeOut = originalSerial.TimeOut;
                replica.Channel = serial;
            }
            else if (channel is Ethernet)
            {
                Ethernet originalEthernet = plcConfig.Channel as Ethernet;
                Ethernet ethernet = new Ethernet();
                ethernet.Protocol = originalEthernet.Protocol;
                ethernet.RemoteIP = originalEthernet.RemoteIP;
                ethernet.RemotePort = originalEthernet.RemotePort;
                ethernet.Retry = originalEthernet.Retry;
                ethernet.TimeOut = originalEthernet.TimeOut;
                replica.Channel = ethernet;
            }
            else if (channel is EthernetListener)
            {
                EthernetListener originalListener = plcConfig.Channel as EthernetListener;
                EthernetListener listener = new EthernetListener();
                listener.LocalPort = originalListener.LocalPort;
                listener.Retry = originalListener.Retry;
                listener.TimeOut = originalListener.TimeOut;
                replica.Channel = listener;
            }

            replica.PlcName = plcConfig.PlcName;
            replica.RequirePlcName = plcConfig.RequirePlcName;
            replica.UnitID = plcConfig.UnitID;

            return replica;
        }

        internal static void SetPlcComConfigFromReplica(PlcComConfig original, PlcComConfig replica)
        {
            original.Channel = replica.Channel;
            original.PlcName = replica.PlcName;
            original.RequirePlcName = replica.RequirePlcName;
            original.UnitID = replica.UnitID;
        }
    }

    internal class ComboItem<T> {

        private string m_Text = string.Empty;
        internal T Value { get; private set; }

        internal ComboItem(string pText, T pValue) {
            Value = pValue;
            m_Text = pText;
        }

        public override string ToString() {
            return m_Text;
        }
    }

    internal class Services {

        internal static void SetComboValue<T>(T pValue,ComboBox pCombo) {

            foreach(ComboItem<T> item in pCombo.Items) {
                if(item.Value.Equals (pValue)) {
                    pCombo.SelectedItem = item;
                    break;
                }
            }
        }

        internal static void FillRetriesCombo(ComboBox pCombo) {

            pCombo.Items.Clear();

            pCombo.Items.Add(new ComboItem<int>("1", 1));
            pCombo.Items.Add(new ComboItem<int>("2", 2));
            pCombo.Items.Add(new ComboItem<int>("3", 3));
            pCombo.Items.Add(new ComboItem<int>("4", 4));
            pCombo.Items.Add(new ComboItem<int>("5", 5));
            pCombo.Items.Add(new ComboItem<int>("8", 8));
            pCombo.Items.Add(new ComboItem<int>("10", 10));
            pCombo.Items.Add(new ComboItem<int>("12", 12));
            pCombo.Items.Add(new ComboItem<int>("15", 15));
        }

        internal static void FillTimeoutCombo(ComboBox pCombo) {

            pCombo.Items.Clear();

            pCombo.Items.Add(new ComboItem<int>("0.5 sec", 500));
            pCombo.Items.Add(new ComboItem<int>("1 sec", 1000));
            pCombo.Items.Add(new ComboItem<int>("1.5 sec", 1500));
            pCombo.Items.Add(new ComboItem<int>("2 sec", 2000));
            pCombo.Items.Add(new ComboItem<int>("3 sec", 3000));
            pCombo.Items.Add(new ComboItem<int>("4 sec", 4000));
            pCombo.Items.Add(new ComboItem<int>("5 sec", 5000));
            pCombo.Items.Add(new ComboItem<int>("6 sec", 6000));
            pCombo.Items.Add(new ComboItem<int>("10 sec", 10000));
            pCombo.Items.Add(new ComboItem<int>("20 sec", 20000));
            pCombo.Items.Add(new ComboItem<int>("30 sec", 30000));
        }

        internal static void FillUnitIDCombo(ComboBox pCombo) {

            pCombo.Items.Clear();
            pCombo.Items.Add(new ComboItem<int>("0   (Direct)", 0));
            for(byte i = 1; i < 128; ++i) {
                string sText = i.ToString().PadRight(4, ' ') + "(CANbus)";
                if(i > 63) {
                    sText = i.ToString().PadRight(4, ' ') + "(RS485)";
                }
                pCombo.Items.Add(new ComboItem<int>(sText, i));
            }
        }

        internal static void FillEthProtocolCombo(ComboBox pCombo) {

            pCombo.Items.Clear();

            pCombo.Items.Add(new ComboItem<EthProtocol>(EthProtocolToString(EthProtocol.TCP), EthProtocol.TCP));
            pCombo.Items.Add(new ComboItem<EthProtocol>(EthProtocolToString(EthProtocol.UDP ), EthProtocol.UDP));
        }

        internal static string EthProtocolToString(EthProtocol pProtType) {
            string result = "Undefined";

            switch(pProtType) {
                case EthProtocol.TCP:
                    result = "TCP";
                    break;
                case EthProtocol.UDP :
                    result = "UDP";
                    break;
            }

            return result;
        }
    }
}
