﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Unitronics.ComConfig.Lite.Presenters;
using Unitronics.ComDriver;
using Unitronics.ComConfig.Lite.Globals;
using Unitronics.ComConfig.Lite.Presenters;

namespace Unitronics.ComConfig.Lite
{
    internal partial class ListenerView : UserControl, IListenerView, IChannelView
    {
        public event EventHandler OnSetAsDirty;
        public event EventHandler OnCheckConnection;

        private Presenters.ListenerPresenter m_Presenter;
        private bool m_IgnoreEvents;

        public ListenerView() {
            InitializeComponent();
            m_Presenter = new ListenerPresenter(this);
            m_Presenter.OnSetAsDirty += new EventHandler(m_Presenter_OnSetAsDirty);
            prvInitGUI();
            SetSettingsToGUI();
        }

        public ListenerView(PlcComConfig plcComConfig) {
            InitializeComponent();
            m_Presenter = new ListenerPresenter(this,plcComConfig);
            m_Presenter.OnSetAsDirty += new EventHandler(m_Presenter_OnSetAsDirty);
            prvInitGUI();
            SetSettingsToGUI();
        }

        #region IListenerView
        public event ValueChangedEventHandler<int> TimeoutChanged;
        public event ValueChangedEventHandler<int> RetriesChanged;
        public event ValueChangedEventHandler<int> EthLocalPortChanged;
        public event EventHandler Loading;

        public void SetSettingsToGUI() {

            m_IgnoreEvents = true;
            txtLocalPort.Text = (m_Presenter.PlcComConfig.Channel as EthernetListener).LocalPort.ToString();
            Services.SetComboValue<int>((m_Presenter.PlcComConfig.Channel as EthernetListener).TimeOut, cmbTimeout);
            Services.SetComboValue<int>((m_Presenter.PlcComConfig.Channel as EthernetListener).Retry, cmbRetries);

            m_IgnoreEvents = false;
        }
        #endregion

        private void prvInitGUI() {

            Services.FillTimeoutCombo(cmbTimeout);
            Services.FillRetriesCombo(cmbRetries);
            fillUnitIDCombo();
        }

        private void cmbRetries_SelectedIndexChanged(object sender, EventArgs e) {

            if(m_IgnoreEvents)
                return;
            ComboItem<int> item = (cmbRetries.SelectedItem as ComboItem<int>);
            ValueChangedEventArgs<int> ea = new ValueChangedEventArgs<int>(item.Value);
            prvRetriesChangedEvent(sender, ea);
        }

        private void cmbTimeout_SelectedIndexChanged(object sender, EventArgs e) {

            if(m_IgnoreEvents)
                return;
            ComboItem<int> item = (cmbTimeout.SelectedItem as ComboItem<int>);
            ValueChangedEventArgs<int> ea = new ValueChangedEventArgs<int>(item.Value);
            prvRaiseTimeoutChangedEvent(sender, ea);
        }

        //private void ListenerView_Load(object sender, EventArgs e) {
        //    if(null != Loading) {
        //        Loading(sender, e);
        //    }
        //}

        private void txtEthPort_Validating(object sender, CancelEventArgs e) {

            if(m_IgnoreEvents) {
                return;
            }
            UInt16 iNewPort;
            if (!UInt16.TryParse(((TextBox)sender).Text, out iNewPort))
            {
                ((TextBox)sender).Text = (m_Presenter.PlcComConfig.Channel as EthernetListener).LocalPort.ToString();
                //e.Cancel = true;
            }
        }

        private void txtEthPort_Validated(object sender, EventArgs e) {

            if(m_IgnoreEvents) {
                return;
            }
            UInt16 iNewPort = UInt16.Parse(((TextBox)sender).Text);
            prvRaiseEthPortChangedEvent(new ValueChangedEventArgs<int>(iNewPort));
        }

        private void prvRaiseEthPortChangedEvent(ValueChangedEventArgs<int> e) {

            if(null != EthLocalPortChanged) {
                EthLocalPortChanged(this, e);
            }
        }

        private void prvRaiseTimeoutChangedEvent(object sender, ValueChangedEventArgs<int> e) {

            if(null != TimeoutChanged) {
                TimeoutChanged(sender, e);
            }
        }

        private void prvRetriesChangedEvent(object sender, ValueChangedEventArgs<int> e) {

            if(null != RetriesChanged) {
                RetriesChanged(sender, e);
            }
        }

        void m_Presenter_OnSetAsDirty(object sender, EventArgs e)
        {
            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }

        #region IChannelView Members

        public PlcComConfig GetPlcComConfig()
        {
            return m_Presenter.PlcComConfig;
        }

        public void ForcePlcName(bool value)
        {
            // Do nothing
        }

        public void SetCheckConnectionEnableState(bool enable)
        {
            btnCheckConnection.Enabled = enable;
        }

        #endregion

        private void btnCheckConnection_Click(object sender, EventArgs e)
        {
            if (OnCheckConnection != null)
            {
                OnCheckConnection(null, null);
            }
        }

        private void cmbUnitID_SelectedIndexChanged(object sender, EventArgs e)
        {
            m_Presenter.PlcComConfig.UnitID = cmbUnitID.SelectedIndex;

            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }

        private void fillUnitIDCombo()
        {
            cmbUnitID.Items.Clear();
            cmbUnitID.Items.Add("0   (Direct)");
            for (int i = 1; i <= 63; i++)
            {
                cmbUnitID.Items.Add(i.ToString().PadRight(4, ' ') + "(CANbus)");
            }

            for (int i = 64; i <= 127; i++)
            {
                cmbUnitID.Items.Add(i.ToString().PadRight(4, ' ') + "(RS485)");
            }

            cmbUnitID.SelectedIndex = m_Presenter.PlcComConfig.UnitID;
        }
    }
}
