﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Xml.Linq;
using System.Xml.Serialization;
using Unitronics.ComDriver;


namespace Unitronics.ComConfig.Lite.Presenters
{
    class FavoritesPresenter
    {
        public event EventHandler OnFavoritesListUpdate;
        private const string FAVS = "Unitronics\\Favorites";
        private const string Favorites_File = "Favorites.xml";
        private List<ComConfig.Channels> m_SupportedChannelsList;
        
        private Favorites m_Favorites;

        public FavoritesPresenter()
        {
            try
            {
                m_Favorites = new Favorites();
                
                if (System.ComponentModel.LicenseManager.UsageMode == System.ComponentModel.LicenseUsageMode.Designtime)
                    return;
                
                string path = Environment.GetFolderPath(Environment.SpecialFolder.CommonProgramFiles);
                string dirSeparator = Path.DirectorySeparatorChar.ToString();
                if (path.Substring(path.Length - 1) == dirSeparator)
                    path = path.Substring(0, path.Length - 1);


                if (!Directory.Exists(path + dirSeparator + "Unitronics"))
                {
                    Directory.CreateDirectory(path + dirSeparator + "Unitronics");
                }

                if (!Directory.Exists(path + dirSeparator + FAVS))
                {
                    Directory.CreateDirectory(path + dirSeparator + FAVS);
                }

                if (File.Exists(path + dirSeparator + FAVS + dirSeparator + Favorites_File))
                {
                    try
                    {
                        m_Favorites = load(path + dirSeparator + FAVS + dirSeparator + Favorites_File);
                    }
                    catch
                    {
                        throw new ApplicationException("Corrupted Favorites XML File. Favorites will not be loaded");
                    }
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        public PlcComConfig PlcComConfig { get; set; }

        public List<ComConfig.Channels> SupportedChannelsList
        {
            set
            {
                List<PlcComConfig> filtereConnections = m_Favorites.FilteredConnections(value);
                m_SupportedChannelsList = value;
                if (OnFavoritesListUpdate != null)
                   OnFavoritesListUpdate(filtereConnections, null);
            }
        }

        public void AddConnectionToFavorites()
        {
            try
            {
                string path = Environment.GetFolderPath(Environment.SpecialFolder.CommonProgramFiles);
                string dirSeparator = Path.DirectorySeparatorChar.ToString();
                if (path.Substring(path.Length - 1) == dirSeparator)
                    path = path.Substring(0, path.Length - 1);

                PlcComConfig plcConfig = Globals.Utils.GetPlcComConfigReplica(PlcComConfig);
                m_Favorites.ConnectionsList.Add(plcConfig);

                List<PlcComConfig> filtereConnections = m_Favorites.FilteredConnections(m_SupportedChannelsList);
                save(path + dirSeparator + FAVS + dirSeparator + Favorites_File, m_Favorites);
                if (OnFavoritesListUpdate != null)
                    OnFavoritesListUpdate(filtereConnections, null);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        public void DeleteConnectionFromFavorites(PlcComConfig plcConfig)
        {
            try
            {
                m_Favorites.ConnectionsList.Remove(plcConfig);
                string path = Environment.GetFolderPath(Environment.SpecialFolder.CommonProgramFiles);
                string dirSeparator = Path.DirectorySeparatorChar.ToString();
                if (path.Substring(path.Length - 1) == dirSeparator)
                    path = path.Substring(0, path.Length - 1);

                save(path + dirSeparator + FAVS + dirSeparator + Favorites_File, m_Favorites);

                List<PlcComConfig> filtereConnections = m_Favorites.FilteredConnections(m_SupportedChannelsList);

                if (OnFavoritesListUpdate != null)
                    OnFavoritesListUpdate(filtereConnections, null);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        public void Save()
        {
            string path = Environment.GetFolderPath(Environment.SpecialFolder.CommonProgramFiles);
            string dirSeparator = Path.DirectorySeparatorChar.ToString();
            if (path.Substring(path.Length - 1) == dirSeparator)
                path = path.Substring(0, path.Length - 1);

            save(path + dirSeparator + FAVS + dirSeparator + Favorites_File, m_Favorites);

            List<PlcComConfig> filtereConnections = m_Favorites.FilteredConnections(m_SupportedChannelsList);

            if (OnFavoritesListUpdate != null)
                OnFavoritesListUpdate(filtereConnections, null);
        }

        //internal PlcComConfig GetPlcComConfigReplica(PlcComConfig plcConfig)
        //{
        //    PlcComConfig replica = new PlcComConfig();
        //    Channel channel = plcConfig.Channel;
        //    if (channel is Serial)
        //    {
        //        Serial serial = new Serial();
        //        Serial originalSerial = (plcConfig.Channel as Serial);
        //        serial.PortName = originalSerial.PortName;
        //        serial.BaudRate = originalSerial.BaudRate;
        //        serial.DataBits = originalSerial.DataBits;
        //        serial.StopBits = originalSerial.StopBits;
        //        serial.Parity = originalSerial.Parity;
        //        serial.Retry = originalSerial.Retry;
        //        serial.TimeOut = originalSerial.TimeOut;
        //        replica.Channel = serial;
        //    }
        //    else if (channel is Ethernet)
        //    {
        //        Ethernet originalEthernet = plcConfig.Channel as Ethernet;
        //        Ethernet ethernet = new Ethernet();
        //        ethernet.Protocol = originalEthernet.Protocol;
        //        ethernet.RemoteIP = originalEthernet.RemoteIP;
        //        ethernet.RemotePort = originalEthernet.RemotePort;
        //        ethernet.Retry = originalEthernet.Retry;
        //        ethernet.TimeOut = originalEthernet.TimeOut;
        //        replica.Channel = ethernet;
        //    }
        //    else if (channel is EthernetListener)
        //    {
        //        EthernetListener originalListener = plcConfig.Channel as EthernetListener;
        //        EthernetListener listener = new EthernetListener();
        //        listener.LocalPort = originalListener.LocalPort;
        //        listener.Retry = originalListener.Retry;
        //        listener.TimeOut = originalListener.TimeOut;
        //        replica.Channel = listener;
        //    }

        //    replica.PlcName = plcConfig.PlcName;
        //    replica.RequirePlcName = plcConfig.RequirePlcName;
        //    replica.UnitID = plcConfig.UnitID;

        //    return replica;
        //}

        //internal void SetPlcComConfigFromReplica(PlcComConfig original, PlcComConfig replica)
        //{
        //    original.Channel = replica.Channel;
        //    original.PlcName = replica.PlcName;
        //    original.RequirePlcName = replica.RequirePlcName;
        //    original.UnitID = replica.UnitID;
        //}

        private Favorites load(string xmlFilePath)
        {
            Favorites favorites;
            using (FileStream fs = new FileStream(xmlFilePath, FileMode.Open))
            {
                XmlSerializer xs = new XmlSerializer(typeof(Favorites));
                favorites = (Favorites)xs.Deserialize(fs);
            }
            return favorites;
        }

        private void save(string xmlFilePath, Favorites favorites)
        {
            using (FileStream fs = new FileStream(xmlFilePath, FileMode.Create))
            {
                XmlSerializer xs = new XmlSerializer(typeof(Favorites));
                xs.Serialize(fs, favorites);
            }
        }
        
    }
}
