﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Unitronics.ComConfig.Lite.Presenters;
using Unitronics.ComDriver;
using System.IO.Ports;
using Unitronics.ComConfig.Lite.Globals;

namespace Unitronics.ComConfig.Lite
{
    internal partial class SerialView : UserControl, IChannelView
    {
        public event EventHandler OnSetAsDirty;
        public event EventHandler OnCheckConnection;

        private Presenters.SerialPresenter m_Presenter;
        public SerialView()
        {
            InitializeComponent();
            m_Presenter = new SerialPresenter();
            fillGui();
        }

        public SerialView(PlcComConfig plcComConfig)
        {
            InitializeComponent();
            m_Presenter = new SerialPresenter(plcComConfig);
            fillGui();
        }

        private void fillGui()
        {
            fillPcPortCombo();
            fillBaudRateCombo();
            Services.FillTimeoutCombo(cmbTimeout);
            Services.SetComboValue<int>(m_Presenter.PlcComConfig.Channel.TimeOut, cmbTimeout);
            fillRetriesCombo();
            fillDataBitsCombo();
            fillStopBitsCombo();
            fillParityCombo();
            fillUnitIDCombo();
            fillPlcNameCheckbox();
            fillPlcName();

            lblPlcName.Enabled = chkRequirePlcName.Checked;
            txtPlcName.Enabled = chkRequirePlcName.Checked;
        }

        private void fillPcPortCombo()
        {
            cmbPort.Items.Clear();
            for (int i = 1; i <= 256; i++)
            {
                cmbPort.Items.Add("COM" + i.ToString());
            }
            cmbPort.SelectedItem = (m_Presenter.PlcComConfig.Channel as Serial).PortName.ToString();
        }

        private void fillBaudRateCombo()
        {
            cmbBaudRate.Items.Clear();
            cmbBaudRate.Items.AddRange( new object[] { 110, 300, 600, 1200, 2400, 4800, 9600, 19200,
                38400, 57600, 115200} );

            cmbBaudRate.SelectedItem = (int)(m_Presenter.PlcComConfig.Channel as Serial).BaudRate;
        }

        //private void fillTimeoutCombo()
        //{
        //    cmbTimeout.Items.Clear();
        //    cmbTimeout.Items.AddRange(new object[] { "0.5 sec", "1 sec", "2 sec", "3 sec", "4 sec", "5 sec", "10 sec", "20 sec", "30 sec" });

        //    for (int i = 0; i < cmbTimeout.Items.Count; i++)
        //    {
        //        int timeout = getTimeoutFromString(cmbTimeout.Items[i].ToString());
        //        if (timeout >= m_Presenter.PlcComConfig.Channel.TimeOut)
        //        {
        //            cmbTimeout.SelectedIndex = i;
        //            return;
        //        }
        //    }
        //    cmbTimeout.SelectedIndex = cmbTimeout.Items.Count - 1;
        //}

        private void fillRetriesCombo()
        {
            cmbRetries.Items.Clear();
            cmbRetries.Items.AddRange(new object[] { 1, 2, 3, 4, 5, 8, 10, 12, 15 });

            for (int i = 0; i < cmbRetries.Items.Count; i++)
            {
                if ((int)cmbRetries.Items[i] >= m_Presenter.PlcComConfig.Channel.Retry)
                {
                    cmbRetries.SelectedIndex = i;
                    return;
                }
            }
            cmbRetries.SelectedIndex = cmbRetries.Items.Count - 1;
        }

        private void fillDataBitsCombo()
        {
            cmbDataBits.Items.Clear();
            cmbDataBits.Items.AddRange(new object[] { 7, 8 });

            cmbDataBits.SelectedItem = (int)(m_Presenter.PlcComConfig.Channel as Serial).DataBits;
        }

        private void fillStopBitsCombo()
        {
            cmbStopBits.Items.Clear();
            cmbStopBits.Items.AddRange(new object[] { StopBits.One, StopBits.Two });

            cmbStopBits.SelectedItem = (m_Presenter.PlcComConfig.Channel as Serial).StopBits;
        }

        private void fillParityCombo()
        {
            cmbParity.Items.Clear();
            cmbParity.Items.AddRange(new object[] { Parity.None, Parity.Odd, Parity.Even, Parity.Mark, Parity.Space });

            cmbParity.SelectedItem = (m_Presenter.PlcComConfig.Channel as Serial).Parity;
        }

        private void fillUnitIDCombo()
        {
            cmbUnitID.Items.Clear();
            cmbUnitID.Items.Add("0   (Direct)");
            for (int i = 1; i <= 63; i++)
            {
                cmbUnitID.Items.Add(i.ToString().PadRight(4,' ') + "(CANbus)");
            }

            for (int i = 64; i <= 127; i++)
            {
                cmbUnitID.Items.Add(i.ToString().PadRight(4, ' ') + "(RS485)");
            }

            cmbUnitID.SelectedIndex = m_Presenter.PlcComConfig.UnitID;        
        }

        private void fillPlcName()
        {
            txtPlcName.Text = m_Presenter.PlcComConfig.PlcName;
        }

        private void fillPlcNameCheckbox()
        {
            chkRequirePlcName.Checked = m_Presenter.PlcComConfig.RequirePlcName;
        }

        private int getTimeoutFromString(string timeout)
        {
            int result = (int)(Convert.ToDecimal(timeout.Replace(" sec", "")) * 1000);
            return result;
        }

        private void cmbPort_SelectedIndexChanged(object sender, EventArgs e)
        {
            Serial oldSerial = (m_Presenter.PlcComConfig.Channel as Serial);
            Serial serial = new Serial((SerialPortNames)Enum.Parse(typeof(SerialPortNames), cmbPort.SelectedItem.ToString()),
                oldSerial.BaudRate, oldSerial.Retry, oldSerial.TimeOut, oldSerial.DataBits, oldSerial.Parity, oldSerial.StopBits);

            m_Presenter.PlcComConfig.Channel = serial;

            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }

        private void cmbBaudRate_SelectedIndexChanged(object sender, EventArgs e)
        {
            (m_Presenter.PlcComConfig.Channel as Serial).BaudRate = (BaudRate)cmbBaudRate.SelectedItem;

            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }

        private void cmbRetries_SelectedIndexChanged(object sender, EventArgs e)
        {
            m_Presenter.PlcComConfig.Channel.Retry = (int)cmbRetries.SelectedItem;

            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }

        //private void cmbTimeout_SelectedIndexChanged(object sender, EventArgs e)
        //{
        //    m_Presenter.PlcComConfig.Channel.TimeOut = getTimeoutFromString(cmbTimeout.SelectedItem.ToString());

        //    if (OnSetAsDirty != null)
        //        OnSetAsDirty(null, null);
        //}

        private void cmbTimeout_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboItem<int> item = (cmbTimeout.SelectedItem as ComboItem<int>);
            m_Presenter.PlcComConfig.Channel.TimeOut = item.Value;

            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }


        private void cmbDataBits_SelectedIndexChanged(object sender, EventArgs e)
        {
            (m_Presenter.PlcComConfig.Channel as Serial).DataBits = (DataBits)cmbDataBits.SelectedItem;

            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }

        private void cmbParity_SelectedIndexChanged(object sender, EventArgs e)
        {
            (m_Presenter.PlcComConfig.Channel as Serial).Parity = (Parity)cmbParity.SelectedItem;

            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }

        private void cmbStopBits_SelectedIndexChanged(object sender, EventArgs e)
        {
            (m_Presenter.PlcComConfig.Channel as Serial).StopBits = (StopBits)cmbStopBits.SelectedItem;

            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }

        private void cmbUnitID_SelectedIndexChanged(object sender, EventArgs e)
        {
            m_Presenter.PlcComConfig.UnitID = cmbUnitID.SelectedIndex;

            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }

        private void chkRequirePlcName_CheckedChanged(object sender, EventArgs e)
        {
            m_Presenter.PlcComConfig.RequirePlcName = chkRequirePlcName.Checked;

            lblPlcName.Enabled = chkRequirePlcName.Checked;
            txtPlcName.Enabled = chkRequirePlcName.Checked;

            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }

        private void txtPlcName_TextChanged(object sender, EventArgs e)
        {
            m_Presenter.PlcComConfig.PlcName = txtPlcName.Text;

            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }

        #region IChannelView Members

        public PlcComConfig GetPlcComConfig()
        {
            return m_Presenter.PlcComConfig;
        }

        public void ForcePlcName(bool value)
        {
            if (value)
            {
                chkRequirePlcName.Checked = true;
                chkRequirePlcName.Enabled = false;
            }
            else
            {
                chkRequirePlcName.Enabled = true;
            }
        }

        public void SetCheckConnectionEnableState(bool enable)
        {
            btnCheckConnection.Enabled = enable;
        }

        #endregion

        private void btnCheckConnection_Click(object sender, EventArgs e)
        {
            if (OnCheckConnection != null)
            {
                OnCheckConnection(null, null);
            }
        }
    }
}
