﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Unitronics.ComDriver;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.Remoting.Messaging;
using System.IO;
using FileTypeAndIcon;

namespace Unitronics.SdCardExplorer.Lite
{
    internal class ExplorerPresenter
    {
        internal class Notifiy
        {
            internal string StatusText;
            internal Color StatusTextColor;
            internal int ProgressMin;
            internal int ProgressMax;
            internal int ProgressValue;
            internal bool ShowProgress;
        }

        internal class GuiEnableState
        {
            internal bool Refresh;
            internal bool Abort;
            internal bool Download;
            internal bool Upload;
            internal bool Copy;
            internal bool Paste;
            internal bool Delete;
            internal bool Folders;
            internal bool Files;
            internal bool Backup;
            internal bool EraseAll;
        }

        private Color ERROR_COLOR = Color.Red;
        private Color NORMAL_COLOR = Color.FromArgb(255, 255, 255, 255);

        internal delegate void downloadFilesAndFoldersDelegate(string currentDir, List<SD.File> files, List<SD.Folder> folders, string destPath, bool allowResume);
        internal delegate void deleteFilesAndFoldersDelegate(string currentDir, List<SD.File> files, List<SD.Folder> folders);
        internal delegate void uploadFilesDelegate(List<string> files, string destPath);
        internal delegate void deleteFilesDelegate(List<string> files);

        internal event EventHandler OnNotify;
        internal event EventHandler OnGuiEnableStateChanged;
        internal event EventHandler OnDirRelist;
        internal event EventHandler OnBusyCursorVisibleChange;

        private PLC m_Plc;
        private Notifiy m_Notifiy;
        private GuiEnableState m_GuiEnableState;
        private SD.Folder m_Folder;
        private bool keepAlive = false;
        private System.Threading.Timer keepAliveTimer;
        private SdCardExplorer m_Owner;

        public ExplorerPresenter(SdCardExplorer owner)
        {
            m_Notifiy = new Notifiy()
            {
                ProgressMax = 100,
                ProgressMin = 0,
                ProgressValue = 0,
                ShowProgress = true,
                StatusText = "",
                StatusTextColor = NORMAL_COLOR,
            };

            m_GuiEnableState = new GuiEnableState()
            {
                Abort = false,
                Copy = false,
                Delete = false,
                Download = false,
                Files = false,
                Folders = false,
                Paste = false,
                Refresh = false,
                Upload = false,
                Backup = false,
                EraseAll = false,
            };

            m_Folder = new SD.Folder();
            HomeDirectory = SD.SdFolder.RootFolder;
            keepAliveTimer = new System.Threading.Timer(connectionKeepAlive, null, 15000, 15000);
            keepAlive = false;
            m_Owner = owner;
        }

        internal List<SD.SdFolder> GetRootChildDirectory()
        {
            List<SD.SdFolder> result = new List<SD.SdFolder>();
            result.Add(SD.SdFolder.DT);
            result.AddRange(GetChildDirectory(SD.SdFolder.DT));

            result.Add(SD.SdFolder.EXCEL);
            result.AddRange(GetChildDirectory(SD.SdFolder.EXCEL));

            result.Add(SD.SdFolder.SDBLOCKS);
            result.Add(SD.SdFolder.SYSTEM);

            result.Add(SD.SdFolder.TRENDS);
            result.AddRange(GetChildDirectory(SD.SdFolder.TRENDS));

            result.Add(SD.SdFolder.USER_APP);
            result.Add(SD.SdFolder.WEB);

            return result;
        }

        internal List<SD.SdFolder> GetChildDirectory(SD.SdFolder parentDir)
        {
            List<SD.SdFolder> result = new List<SD.SdFolder>();
            switch (parentDir)
            {
                case SD.SdFolder.DT:
                    result.Add(SD.SdFolder.DT_DT1);
                    result.Add(SD.SdFolder.DT_DT2);
                    result.Add(SD.SdFolder.DT_DT3);
                    result.Add(SD.SdFolder.DT_DT4);
                    break;
                case SD.SdFolder.EXCEL:
                    result.Add(SD.SdFolder.EXCEL_EXCEL1);
                    result.Add(SD.SdFolder.EXCEL_EXCEL2);
                    result.Add(SD.SdFolder.EXCEL_EXCEL3);
                    result.Add(SD.SdFolder.EXCEL_EXCEL4);
                    break;
                case SD.SdFolder.TRENDS:
                    result.Add(SD.SdFolder.TRENDS_TRENDS1);
                    result.Add(SD.SdFolder.TRENDS_TRENDS2);
                    result.Add(SD.SdFolder.TRENDS_TRENDS3);
                    result.Add(SD.SdFolder.TRENDS_TRENDS4);
                    break;
            }

            return result;
        }

        internal PLC Plc
        {
            set
            {
                m_Plc = value;
                if (m_Plc != null)
                {
                    ReloadFilesList();
                }
                else
                {
                    // Clear view
                    m_Folder = new SD.Folder();
                    DirRelist();

                    GuiEnableStateChanged();
                }
            }
            get
            {
                return m_Plc;
            }

        }

        internal SD.SdFolder HomeDirectory { set; get; }

        internal bool isUnitronicsDirectory(string path)
        {
            string dir = GetFullPathFromHomeDirAndPath(HomeDirectory, path);

            SD.SdFolder sdFolder = SD.GetSdFolderFromString(dir);
            if (sdFolder == SD.SdFolder.RootFolder)
            {
                return false;
            }
            else
            {
                return true;
            }
            
        }

        internal void ReloadFilesList()
        {
            if (Plc == null)
            {
                m_Notifiy.StatusText = "PLC was not set";
                m_Notifiy.StatusTextColor = ERROR_COLOR;
                notify();              
                return;
            }

            MethodInvoker methodInvoker = new MethodInvoker(dirAllSubFoldersAndFiles);
            methodInvoker.BeginInvoke(dirCompleted, null);
        }

        internal void Abort()
        {
            if (Plc == null)
            {
                m_Notifiy.StatusText = "PLC was not set";
                m_Notifiy.StatusTextColor = ERROR_COLOR;
                notify();
                return;
            }

            Plc.Abort();
        }

        internal void DownloadFilesAndFolders(string currentDir, List<SD.File> files, List<SD.Folder> folders, string destPath, bool allowResume, MethodInvoker doOnDownloadComplete)
        {
            if ((files.Count == 0) && (folders.Count == 0))
                return;

            if (Plc == null)
            {
                m_Notifiy.StatusText = "PLC was not set";
                m_Notifiy.StatusTextColor = ERROR_COLOR;
                notify();
                return;
            }

            downloadFilesAndFoldersDelegate del = new downloadFilesAndFoldersDelegate(downloadFilesAndFolders);
            del.BeginInvoke(currentDir, files, folders, destPath, allowResume, downloadCompleted, doOnDownloadComplete);
        }

        internal void DeleteFilesAndFolders(string currentDir, List<SD.File> files, List<SD.Folder> folders)
        {
            if ((files.Count == 0) && (folders.Count == 0))
                return;

            if (Plc == null)
            {
                m_Notifiy.StatusText = "PLC was not set";
                m_Notifiy.StatusTextColor = ERROR_COLOR;
                notify();
                return;
            }

            deleteFilesAndFoldersDelegate del = new deleteFilesAndFoldersDelegate(deleteFilesAndFolders);
            del.BeginInvoke(currentDir, files, folders, deleteAllFilesCompleted, null);
        }

        internal void UploadFiles(List<string> files, string destPath)
        {
            if (files.Count == 0)
                return;

            if (Plc == null)
            {
                m_Notifiy.StatusText = "PLC was not set";
                m_Notifiy.StatusTextColor = ERROR_COLOR;
                notify();
                return;
            }

            uploadFilesDelegate del = new uploadFilesDelegate(uploadFiles);
            del.BeginInvoke(files, destPath, uploadCompleted, null);
        }

        internal void DeleteFiles(List<string> files)
        {
            if (files.Count == 0)
                return;

            if (Plc == null)
            {
                m_Notifiy.StatusText = "PLC was not set";
                m_Notifiy.StatusTextColor = ERROR_COLOR;
                notify();
                return;
            }

            deleteFilesDelegate del = new deleteFilesDelegate(deleteFiles);
            del.BeginInvoke(files, deleteCompleted, null);
        }

        internal void CopyFiles(List<string> files)
        {
            if (files.Count == 0)
                return;

            if (Plc == null)
            {
                m_Notifiy.StatusText = "PLC was not set";
                m_Notifiy.StatusTextColor = ERROR_COLOR;
                notify();
                return;
            }
        }

        internal void PasteFiles(List<string> files, string destPath)
        {
            if (files.Count == 0)
                return;

            if (Plc == null)
            {
                m_Notifiy.StatusText = "PLC was not set";
                m_Notifiy.StatusTextColor = ERROR_COLOR;
                notify();
                return;
            }
        }

        internal SD.Folder GetSdFolderFromPath(string path)
        {
            SD.Folder folder = new SD.Folder();
            string[] directories = path.Split('\\');
            SD.Folder currentDir = m_Folder;

            for (int i = 0; i < directories.Length; i++)
            {
                SD.Folder resultFolder;
                if (searchForSubFolder(currentDir, directories[i], out resultFolder))
                {
                    currentDir = resultFolder;
                }
                else
                {
                    return null;
                }
            }

            return currentDir;
        }

        internal static string GetFullPathFromHomeDirAndPath(SD.SdFolder homeDir, string path)
        {
            string dir = SD.GetSdFolderAsString(homeDir);
            // Home directory is not root, then take all path except the last Dir, for example:
            // This\Is\A\Path\Hello\World
            // We take only This\Is\A\Path\Hello\  since "World" will be the Dir (the only dir) on the tree.

            dir = "\\" + dir;
            int lastIndexOfDirSeperator = dir.LastIndexOf('\\');
            dir = dir.Substring(0, lastIndexOfDirSeperator);

            dir += "\\" + path;

            if (dir[0] == '\\')
            {
                dir = dir.Substring(1);
            }

            return dir;
        }

        internal static string FileSizeAsString(uint fileSize)
        {
            uint fileSizeInKB = fileSize / 1024;
            if ((fileSize % 1024) != 0)
                fileSizeInKB += 1;

            return fileSizeInKB.ToString("###,###,##0") + " KB";
        }

        protected void UpdateUI(MethodInvoker uiDelegate)
        {
            if (m_Owner.InvokeRequired)
                m_Owner.Invoke(uiDelegate);
            else
                uiDelegate();
        }

        private bool searchForSubFolder(SD.Folder folderToSearchIn, string subFolder, out SD.Folder result)
        {
            foreach (SD.Folder folder in folderToSearchIn.Folders)
            {
                if (folder.Name == subFolder)
                {
                    result = folder;
                    return true;
                }
            }
            result = null;
            return false;
        }

        private void notify()
        {
            if (OnNotify != null)
                OnNotify(m_Notifiy, null);
        }

        private void GuiEnableStateChanged()
        {
            if (m_Plc == null)
            {
                m_GuiEnableState.Abort = false;
                m_GuiEnableState.Copy = false;
                m_GuiEnableState.Delete = false;
                m_GuiEnableState.Download = false;
                m_GuiEnableState.Files = false;
                m_GuiEnableState.Folders = false;
                m_GuiEnableState.Paste = false;
                m_GuiEnableState.Refresh = false;
                m_GuiEnableState.Upload = false;
                m_GuiEnableState.Backup = false;
                m_GuiEnableState.EraseAll = false;
                keepAlive = false;
            }

            if (OnGuiEnableStateChanged != null)
                OnGuiEnableStateChanged(m_GuiEnableState, null);
        }

        private void DirRelist()
        {
            if (OnDirRelist != null)
                OnDirRelist(m_Folder, null);
        }

        private void BusyCursorVisibleChange(bool value)
        {
            if (OnBusyCursorVisibleChange != null)
                OnBusyCursorVisibleChange(value, null);
        }

        private void dirAllSubFoldersAndFiles()
        {
            m_Notifiy.StatusText = "Listing Folders & Files";
            m_Notifiy.StatusTextColor = NORMAL_COLOR;
            m_Notifiy.ProgressMin = 0;
            m_Notifiy.ProgressMax = 100;
            m_Notifiy.ProgressValue = 0;
            m_GuiEnableState.Copy = false;
            m_GuiEnableState.Delete = false;
            m_GuiEnableState.Download = false;
            m_GuiEnableState.Files = false;
            m_GuiEnableState.Folders = false;
            m_GuiEnableState.Paste = false;
            m_GuiEnableState.Upload = false;
            m_GuiEnableState.Refresh = false;
            m_GuiEnableState.Backup = false;
            m_GuiEnableState.EraseAll = false;
            m_GuiEnableState.Abort = true;
            keepAlive = false;
           
            notify();
            GuiEnableStateChanged();
            BusyCursorVisibleChange(true);

            ProgressStatusChangedDelegate progressStatusDelegate = new ProgressStatusChangedDelegate(progressStatusChanged);

            string dir = SD.GetSdFolderAsString(HomeDirectory);

            if (dir == "")
            {
                m_Folder = Plc.SD.Dir(progressStatusDelegate);
            }
            else
            {
                m_Folder = new SD.Folder();
                SD.Folder folder = new SD.Folder();
                folder = Plc.SD.Dir(HomeDirectory, "*.*", true, progressStatusDelegate);

                m_Folder.Folders.Add(folder);
            }
            DirRelist();
        }

        private void reDirSpecificFolder(string path)
        {
            if (path == null)
            {
                dirAllSubFoldersAndFiles();
                return;
            }

            m_Notifiy.StatusText = "Listing Folders & Files";
            m_Notifiy.StatusTextColor = NORMAL_COLOR;
            m_Notifiy.ProgressMin = 0;
            m_Notifiy.ProgressMax = 100;
            m_Notifiy.ProgressValue = 0;
            m_GuiEnableState.Copy = false;
            m_GuiEnableState.Delete = false;
            m_GuiEnableState.Download = false;
            m_GuiEnableState.Files = false;
            m_GuiEnableState.Folders = false;
            m_GuiEnableState.Paste = false;
            m_GuiEnableState.Upload = false;
            m_GuiEnableState.Refresh = false;
            m_GuiEnableState.Backup = false;
            m_GuiEnableState.EraseAll = false;
            m_GuiEnableState.Abort = true;
            keepAlive = false;

            notify();
            GuiEnableStateChanged();
            BusyCursorVisibleChange(true);

            ProgressStatusChangedDelegate progressStatusDelegate = new ProgressStatusChangedDelegate(progressStatusChanged);

            string dir = SD.GetSdFolderAsString(HomeDirectory);

            SD.Folder folder = Plc.SD.Dir(path, "*.*", false, progressStatusDelegate);
            SD.Folder oldFolder;

            oldFolder = GetSdFolderFromPath(path);

            oldFolder.Files.Clear();
            oldFolder.Files.AddRange(folder.Files);
            
            DirRelist();
        }

        private void deleteFilesAndFolders(string currentDir, List<SD.File> files, List<SD.Folder> folders)
        {
            m_Notifiy.StatusText = "Deleting All Files From SD";
            m_Notifiy.StatusTextColor = NORMAL_COLOR;
            m_Notifiy.ProgressMin = 0;
            m_Notifiy.ProgressMax = 100;
            m_Notifiy.ProgressValue = 0;
            m_GuiEnableState.Copy = false;
            m_GuiEnableState.Delete = false;
            m_GuiEnableState.Download = false;
            m_GuiEnableState.Files = false;
            m_GuiEnableState.Folders = false;
            m_GuiEnableState.Paste = false;
            m_GuiEnableState.Upload = false;
            m_GuiEnableState.Refresh = false;
            m_GuiEnableState.Backup = false;
            m_GuiEnableState.EraseAll = false;
            m_GuiEnableState.Abort = true;
            keepAlive = false;

            notify();
            GuiEnableStateChanged();
            BusyCursorVisibleChange(true);

            deleteFiles(currentDir, files);
            deleteAllFilesFromFolder(currentDir, folders);

            dirAllSubFoldersAndFiles();
            DirRelist();
        }

        // Use by the function for deleting all files from SD
        private void deleteFiles(string currentDir, List<SD.File> files)
        {
            ProgressStatusChangedDelegate progressStatusDelegate = new ProgressStatusChangedDelegate(progressStatusChanged);

            if (currentDir.Length > 0 && currentDir[0] == '\\')
            {
                currentDir = currentDir.Substring(1);
            }

            foreach (SD.File file in files)
            {
                Plc.SD.DeleteFile(currentDir, file.Name, progressStatusDelegate);
            }
        }

        private void deleteAllFilesFromFolder(string currentDir, List<SD.Folder> folders)
        {
            foreach (SD.Folder folder in folders)
            {
                deleteFiles(currentDir + "\\" + folder.Name, folder.Files);
                deleteAllFilesFromFolder(currentDir + "\\" + folder.Name, folder.Folders);
            }
        }

        private void downloadFilesAndFolders(string currentDir, List<SD.File> files, List<SD.Folder> folders, string destPath, bool allowResume)
        {
            m_Notifiy.StatusText = "Reading Folders & Files";
            m_Notifiy.StatusTextColor = NORMAL_COLOR;
            m_Notifiy.ProgressMin = 0;
            m_Notifiy.ProgressMax = 100;
            m_Notifiy.ProgressValue = 0;
            m_GuiEnableState.Copy = false;
            m_GuiEnableState.Delete = false;
            m_GuiEnableState.Download = false;
            m_GuiEnableState.Files = false;
            m_GuiEnableState.Folders = false;
            m_GuiEnableState.Paste = false;
            m_GuiEnableState.Upload = false;
            m_GuiEnableState.Refresh = false;
            m_GuiEnableState.Backup = false;
            m_GuiEnableState.EraseAll = false;
            m_GuiEnableState.Abort = true;
            keepAlive = false;

            notify();
            GuiEnableStateChanged();
            BusyCursorVisibleChange(true);

            frmConfirmFileReplace.ConfirmReplaceButtons confirmReplaceResult = frmConfirmFileReplace.ConfirmReplaceButtons.None;

            downloadFile(currentDir, files, destPath, allowResume, ref confirmReplaceResult);
            downloadFolder(currentDir, folders, destPath, allowResume, ref confirmReplaceResult);
        }

        private void downloadFile(string currentDir, List<SD.File> files, string destPath, bool allowResume, 
            ref frmConfirmFileReplace.ConfirmReplaceButtons confirmReplaceResult)
        {
            ProgressStatusChangedDelegate progressStatusDelegate = new ProgressStatusChangedDelegate(progressStatusChanged);

            if (destPath[destPath.Length - 1] == '\\')
            {
                destPath = destPath.Substring(0,destPath.Length - 1);
            }

            if (currentDir.Length > 0 && currentDir[0] == '\\')
            {
                currentDir = currentDir.Substring(1);
            }

            if (!System.IO.Directory.Exists(destPath))
            {
                System.IO.Directory.CreateDirectory(destPath);
            }

            foreach (SD.File file in files)
            {
                if (allowResume)
                {
                    if (System.IO.File.Exists(destPath + "\\" + file.Name))
                    {
                        FileInfo fi = new FileInfo(destPath + "\\" + file.Name);
                        long fileLen = fi.Length;
                        int buttonsToShow = 0;
                        if (fileLen < file.Size)
                        {
                            buttonsToShow = 127;
                        }
                        else
                        {
                            buttonsToShow = 123; // Resume is disabled
                        }

                        if (confirmReplaceResult == frmConfirmFileReplace.ConfirmReplaceButtons.None)
                        {
                            frmConfirmFileReplace confirmReplace = new frmConfirmFileReplace();
                            confirmReplace.FileName = file.Name;
                            confirmReplace.SizeExisting = fileLen;
                            confirmReplace.DateExisting = fi.LastWriteTime.ToString();
                            confirmReplace.SizeNew = file.Size;
                            confirmReplace.DateNew = file.DateModified.ToString();
                            confirmReplace.enabledButtons = buttonsToShow;

                            Icon[] icons = RegisteredFileType.GetIconFromExtension(System.IO.Path.GetExtension(file.Name));
                            
                            if ((icons == null) || (icons[0] == null) || (icons[1] == null))
                                icons = RegisteredFileType.GetShellIconsFromExtension(System.IO.Path.GetExtension(file.Name));

                            if (icons != null && icons[0] != null && icons[1] != null)
                            {
                                confirmReplace.FileIcon = icons[0].ToBitmap();
                            }
                            else
                            {
                                confirmReplace.FileIcon = null;
                            }

                            if (icons != null)
                            {
                                confirmReplace.FileIcon = icons[0].ToBitmap();
                            }
                            else
                            {
                                confirmReplace.FileIcon = null;
                            }

                            MethodInvoker uiDelegate = delegate
                            {
                                confirmReplace.ShowDialog(m_Owner);  
                            };
                            UpdateUI(uiDelegate);
                            
                            confirmReplaceResult = confirmReplace.Result;

                            if (confirmReplaceResult == frmConfirmFileReplace.ConfirmReplaceButtons.Cancel)
                                break;
                        }
                        if (confirmReplaceResult == frmConfirmFileReplace.ConfirmReplaceButtons.Skip ||
                            confirmReplaceResult == frmConfirmFileReplace.ConfirmReplaceButtons.SkipAll)
                        {
                            // Do nothing (Skip file)
                        }
                        else if (confirmReplaceResult == frmConfirmFileReplace.ConfirmReplaceButtons.Resume ||
                            confirmReplaceResult == frmConfirmFileReplace.ConfirmReplaceButtons.ResumeAll)
                        {
                            if (fileLen < file.Size)
                            {
                                Plc.SD.ReadFile(currentDir, file.Name, destPath, true, progressStatusDelegate);
                            }
                            else
                            {
                                // Skip file
                            }
                        }
                        else if (confirmReplaceResult == frmConfirmFileReplace.ConfirmReplaceButtons.Overwrite ||
                            confirmReplaceResult == frmConfirmFileReplace.ConfirmReplaceButtons.OverwriteAll)
                        {
                            Plc.SD.ReadFile(currentDir, file.Name, destPath, false, progressStatusDelegate);
                        }
                    }
                    else
                    {
                        Plc.SD.ReadFile(currentDir, file.Name, destPath, false, progressStatusDelegate);
                    }
                }
                else
                {
                    Plc.SD.ReadFile(currentDir, file.Name, destPath, false, progressStatusDelegate);
                }

                if (confirmReplaceResult == frmConfirmFileReplace.ConfirmReplaceButtons.Overwrite ||
                confirmReplaceResult == frmConfirmFileReplace.ConfirmReplaceButtons.Resume ||
                confirmReplaceResult == frmConfirmFileReplace.ConfirmReplaceButtons.Skip)
                {
                    // Reset state for next file;
                    confirmReplaceResult = frmConfirmFileReplace.ConfirmReplaceButtons.None;
                }
            }
        }

        private void downloadFolder(string currentDir, List<SD.Folder> folders, string destPath, bool allowResume, ref frmConfirmFileReplace.ConfirmReplaceButtons confirmReplaceResult)
        {
            foreach (SD.Folder folder in folders)
            {
                downloadFile(currentDir + "\\" + folder.Name, folder.Files, destPath + "\\" + folder.Name, allowResume, ref confirmReplaceResult);
                downloadFolder(currentDir + "\\" + folder.Name, folder.Folders, destPath + "\\" + folder.Name, allowResume, ref confirmReplaceResult);
            }
        }

        private void uploadFiles(List<string> files, string destPath)
        {
            int numOfUploadedFiles = 0;
            if (!isUnitronicsDirectory(destPath)) 
                return;

            m_Notifiy.StatusText = "Writting Files to SD";
            m_Notifiy.StatusTextColor = NORMAL_COLOR;
            m_Notifiy.ProgressMin = 0;
            m_Notifiy.ProgressMax = 100;
            m_Notifiy.ProgressValue = 0;
            m_GuiEnableState.Copy = false;
            m_GuiEnableState.Delete = false;
            m_GuiEnableState.Download = false;
            m_GuiEnableState.Files = false;
            m_GuiEnableState.Folders = false;
            m_GuiEnableState.Paste = false;
            m_GuiEnableState.Upload = false;
            m_GuiEnableState.Refresh = false;
            m_GuiEnableState.Backup = false;
            m_GuiEnableState.EraseAll = false;
            m_GuiEnableState.Abort = true;
            keepAlive = false;

            notify();
            GuiEnableStateChanged();
            BusyCursorVisibleChange(true);

            ProgressStatusChangedDelegate progressStatusDelegate = new ProgressStatusChangedDelegate(progressStatusChanged);

            SD.SdFolder sdFolder = SD.GetSdFolderFromString(destPath);

            foreach (string file in files)
            {
                string extention = getExtention(file).ToUpper();

                if (allowedExtention(sdFolder,extention))
                {
                    numOfUploadedFiles++;
                    byte[] binData = System.IO.File.ReadAllBytes(file);
                    string filenameWithoutPath = System.IO.Path.GetFileName(file);
                    Plc.SD.WriteFile(sdFolder, filenameWithoutPath, binData, progressStatusDelegate);
                }
            }

            if (numOfUploadedFiles > 0)
            {
                reDirSpecificFolder(destPath);
                DirRelist();
            }

            if (numOfUploadedFiles != 1)
            {
                m_Notifiy.StatusText = numOfUploadedFiles.ToString() + " files were written to SD Card.";
            }
            else
            {
                m_Notifiy.StatusText = numOfUploadedFiles.ToString() + " file was written to SD Card.";
            }

            if (numOfUploadedFiles != files.Count())
            {
                int numOfIgnoredFiles = files.Count() - numOfUploadedFiles;
                if (numOfIgnoredFiles != 1)
                {
                    m_Notifiy.StatusText += " " + (numOfIgnoredFiles).ToString() + " files were ignored (allowed files extensions: " + allowedExtentions(sdFolder) + ").";
                }
                else
                {
                    m_Notifiy.StatusText += " " + (numOfIgnoredFiles).ToString() + " file was ignored (allowed files extensions: " + allowedExtentions(sdFolder) + ").";
                }
            }
            m_Notifiy.StatusTextColor = NORMAL_COLOR;
            notify();
        }

        private void deleteFiles(List<string> files)
        {
            m_Notifiy.StatusText = "Deleting Files";
            m_Notifiy.StatusTextColor = NORMAL_COLOR;
            m_Notifiy.ProgressMin = 0;
            m_Notifiy.ProgressMax = 100;
            m_Notifiy.ProgressValue = 0;
            m_GuiEnableState.Copy = false;
            m_GuiEnableState.Delete = false;
            m_GuiEnableState.Download = false;
            m_GuiEnableState.Files = false;
            m_GuiEnableState.Folders = false;
            m_GuiEnableState.Paste = false;
            m_GuiEnableState.Upload = false;
            m_GuiEnableState.Refresh = false;
            m_GuiEnableState.Backup = false;
            m_GuiEnableState.EraseAll = false;
            m_GuiEnableState.Abort = true;
            keepAlive = false;

            notify();
            GuiEnableStateChanged();
            BusyCursorVisibleChange(true);

            ProgressStatusChangedDelegate progressStatusDelegate = new ProgressStatusChangedDelegate(progressStatusChanged);
            string folder = "";

            foreach (string file in files)
            {
                int lastIndexOfPathDelimiter = file.LastIndexOf('\\');
                folder = file.Remove(lastIndexOfPathDelimiter);
                string fileWithoutPath = file.Substring(lastIndexOfPathDelimiter + 1);
                Plc.SD.DeleteFile(folder, fileWithoutPath, progressStatusDelegate);
            }

            reDirSpecificFolder(folder);

            DirRelist();
        }

        private void dirCompleted(IAsyncResult result)
        {
            try
            {
                AsyncResult async;
                async = (AsyncResult)result;
                MethodInvoker del = (MethodInvoker)async.AsyncDelegate;
                del.EndInvoke(result);

                m_Notifiy.StatusText = "";

            }
            catch (Exception ex)
            {
                if (m_Folder == null)
                {
                    m_Folder = new SD.Folder();
                }
                m_Notifiy.StatusText = ex.Message;
                m_Notifiy.StatusTextColor = ERROR_COLOR;
                DirRelist();
            }
            finally
            {
                m_GuiEnableState.Copy = true;
                m_GuiEnableState.Delete = true;
                m_GuiEnableState.Download = true;
                m_GuiEnableState.Files = true;
                m_GuiEnableState.Folders = true;
                m_GuiEnableState.Paste = true;
                m_GuiEnableState.Upload = true;
                m_GuiEnableState.Refresh = true;
                m_GuiEnableState.Backup = true;
                m_GuiEnableState.EraseAll = true;
                m_GuiEnableState.Abort = false;
                keepAlive = true;
                notify();
                GuiEnableStateChanged();
                BusyCursorVisibleChange(false);
            }
        }

        private void uploadCompleted(IAsyncResult result)
        {
            try
            {
                AsyncResult async;
                async = (AsyncResult)result;
                uploadFilesDelegate del = (uploadFilesDelegate)async.AsyncDelegate;
                del.EndInvoke(result);
            }
            catch (Exception ex)
            {
                m_Notifiy.StatusText = ex.Message;
                m_Notifiy.StatusTextColor = ERROR_COLOR;
            }
            finally
            {
                m_GuiEnableState.Copy = true;
                m_GuiEnableState.Delete = true;
                m_GuiEnableState.Download = true;
                m_GuiEnableState.Files = true;
                m_GuiEnableState.Folders = true;
                m_GuiEnableState.Paste = true;
                m_GuiEnableState.Upload = true;
                m_GuiEnableState.Refresh = true;
                m_GuiEnableState.Backup = true;
                m_GuiEnableState.EraseAll = true;
                m_GuiEnableState.Abort = false;
                keepAlive = true;
                notify();
                GuiEnableStateChanged();
                BusyCursorVisibleChange(false);
            }
        }

        private void downloadCompleted(IAsyncResult result)
        {
            MethodInvoker onDownloadComplete = null;
            try
            {
                AsyncResult async;
                async = (AsyncResult)result;
                downloadFilesAndFoldersDelegate del = (downloadFilesAndFoldersDelegate)async.AsyncDelegate;
                onDownloadComplete = (MethodInvoker)async.AsyncState;
                del.EndInvoke(result);

                m_Notifiy.StatusText = "";           
            }
            catch (Exception ex)
            {
                m_Notifiy.StatusText = ex.Message;
                m_Notifiy.StatusTextColor = ERROR_COLOR;
            }
            finally
            {
                m_GuiEnableState.Copy = true;
                m_GuiEnableState.Delete = true;
                m_GuiEnableState.Download = true;
                m_GuiEnableState.Files = true;
                m_GuiEnableState.Folders = true;
                m_GuiEnableState.Paste = true;
                m_GuiEnableState.Upload = true;
                m_GuiEnableState.Refresh = true;
                m_GuiEnableState.Backup = true;
                m_GuiEnableState.EraseAll = true;
                m_GuiEnableState.Abort = false;
                keepAlive = true;
                notify();
                GuiEnableStateChanged();
                BusyCursorVisibleChange(false);

                if (onDownloadComplete != null)
                    onDownloadComplete.BeginInvoke(null, null);
            }
        }

        private void deleteAllFilesCompleted(IAsyncResult result)
        {
            try
            {
                AsyncResult async;
                async = (AsyncResult)result;
                deleteFilesAndFoldersDelegate del = (deleteFilesAndFoldersDelegate)async.AsyncDelegate;
                del.EndInvoke(result);

                m_Notifiy.StatusText = "";

            }
            catch (Exception ex)
            {
                m_Notifiy.StatusText = ex.Message;
                m_Notifiy.StatusTextColor = ERROR_COLOR;
            }
            finally
            {
                m_GuiEnableState.Copy = true;
                m_GuiEnableState.Delete = true;
                m_GuiEnableState.Download = true;
                m_GuiEnableState.Files = true;
                m_GuiEnableState.Folders = true;
                m_GuiEnableState.Paste = true;
                m_GuiEnableState.Upload = true;
                m_GuiEnableState.Refresh = true;
                m_GuiEnableState.Backup = true;
                m_GuiEnableState.EraseAll = true;
                m_GuiEnableState.Abort = false;
                keepAlive = true;
                notify();
                GuiEnableStateChanged();
                BusyCursorVisibleChange(false);
            }
        }

        private void deleteCompleted(IAsyncResult result)
        {
            try
            {
                AsyncResult async;
                async = (AsyncResult)result;
                deleteFilesDelegate del = (deleteFilesDelegate)async.AsyncDelegate;
                del.EndInvoke(result);

                m_Notifiy.StatusText = "";

            }
            catch (Exception ex)
            {
                m_Notifiy.StatusText = ex.Message;
                m_Notifiy.StatusTextColor = ERROR_COLOR;
            }
            finally
            {
                m_GuiEnableState.Copy = true;
                m_GuiEnableState.Delete = true;
                m_GuiEnableState.Download = true;
                m_GuiEnableState.Files = true;
                m_GuiEnableState.Folders = true;
                m_GuiEnableState.Paste = true;
                m_GuiEnableState.Upload = true;
                m_GuiEnableState.Refresh = true;
                m_GuiEnableState.Backup = true;
                m_GuiEnableState.EraseAll = true;
                m_GuiEnableState.Abort = false;
                keepAlive = true;
                notify();
                GuiEnableStateChanged();
                BusyCursorVisibleChange(false);
            }
        }

        private string getExtention(string filename)
        {
            string[] dotDelimited = filename.Split('.');
            return dotDelimited[dotDelimited.Length - 1];
        }

        private string allowedExtentions(SD.SdFolder sdFolder)
        {
            switch (sdFolder)
            {
                case SD.SdFolder.ALARMS:
                    return "UAL";
                    break;
                case SD.SdFolder.DT:
                case SD.SdFolder.DT_DT1:
                case SD.SdFolder.DT_DT2:
                case SD.SdFolder.DT_DT3:
                case SD.SdFolder.DT_DT4:
                    return "UDT, FDT";
                    break;
                case SD.SdFolder.LOG:
                    return "ULG";
                    break;
                case SD.SdFolder.EXCEL:
                case SD.SdFolder.EXCEL_EXCEL1:
                case SD.SdFolder.EXCEL_EXCEL2:
                case SD.SdFolder.EXCEL_EXCEL3:
                case SD.SdFolder.EXCEL_EXCEL4:
                    return "CSV, TXT";
                    break;
                case SD.SdFolder.TRENDS:
                case SD.SdFolder.TRENDS_TRENDS1:
                case SD.SdFolder.TRENDS_TRENDS2:
                case SD.SdFolder.TRENDS_TRENDS3:
                case SD.SdFolder.TRENDS_TRENDS4:
                    return "UTR";
                    break;
                case SD.SdFolder.SDBLOCKS:
                    return "UDB";
                    break;
                case SD.SdFolder.SYSTEM:
                    return "O13, O35, O57, O10, C13, C35, C57, C10";
                    break;
                case SD.SdFolder.USER_APP:
                    return "V13, V35, V57, V10";
                    break;
                case SD.SdFolder.WEB:
                    return "*.*";
                    break;
                default:
                    return "";
            }
        }
        private bool allowedExtention(SD.SdFolder sdFolder, string extention)
        {
            string[] allowedExt = allowedExtentions(sdFolder).Split(new char[] { ',', ' ' } ,StringSplitOptions.RemoveEmptyEntries);
            if (allowedExt.Contains("*.*"))
            {
                return true;
            }
            else if (allowedExt.Contains(extention))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private void progressStatusChanged(RequestProgress requestProgree)
        {
            switch (requestProgree.NotificationType)
            {
                case RequestProgress.en_NotificationType.SetMinMax:
                    m_Notifiy.ProgressMin = requestProgree.Minimum;
                    m_Notifiy.ProgressMax = requestProgree.Maximum;
                    m_Notifiy.ProgressValue = requestProgree.Value;
                    m_Notifiy.StatusText = requestProgree.Text;
                    break;
                case RequestProgress.en_NotificationType.ProgressChanged:
                    m_Notifiy.ProgressValue = requestProgree.Value;
                    m_Notifiy.StatusText = requestProgree.Text;
                    break;
                case RequestProgress.en_NotificationType.Completed:
                    m_Notifiy.ProgressValue = requestProgree.Maximum;
                    m_Notifiy.StatusText = requestProgree.Text;
                    break;
            }
            notify();
        }

        private void connectionKeepAlive(object source)
        {
            try
            {
                if (keepAlive && m_Plc != null && m_Plc.PLCChannel != null && m_Plc.PLCChannel.Connected)
                {
                    string noop = Plc.SendString("ID");
                }
            }
            catch
            {
            }
        }
    }
}
