﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;

namespace Unitronics.SdCardExplorer.Lite
{
    internal class TransparentControl : Control
    {
        private readonly System.Threading.Timer refresher;
        private Image _image;
        private int _frameIndex;
        public TransparentControl()
        {
            SetStyle(ControlStyles.SupportsTransparentBackColor, true);
            BackColor = Color.Transparent;
            refresher = new System.Threading.Timer(TimerOnTick,null,100,100);
            _frameIndex = 0;
        }
        protected override CreateParams CreateParams
        {
            get
            {
                CreateParams cp = base.CreateParams;
                cp.ExStyle |= 0x20;
                return cp;
            }
        }
        protected override void OnMove(EventArgs e)
        {
            RecreateHandle();
        }
        protected override void OnPaint(PaintEventArgs e)
        {
            if (!this.Visible)
                return;

            drawImage(e.Graphics);
            if (this.Text != null)
            {
                Font font = this.Font;
                Size stringSize = getStringSize(this.Text, font, e.Graphics);
                e.Graphics.DrawString(this.Text, font, new SolidBrush(this.ForeColor), (Width / 2) - (stringSize.Width / 2), (Height / 2) - (stringSize.Height / 2),StringFormat.GenericTypographic);
            }
        }
        protected override void OnPaintBackground(PaintEventArgs e)
        { 
            //Do not paint background 
        }

        //Hack
        public void Redraw()
        {
            RecreateHandle();
        }
        private void TimerOnTick(object source)
        {
            _frameIndex++;
            Invalidate();
        }
        public Image Image
        {
            get 
            {
                return _image;
            }
            set
            {
                _image = value;
                RecreateHandle();
            }
        }

        private void drawImage(Graphics g)
        {
            if (_image != null)
            {
                FrameDimension dimension = new FrameDimension(_image.FrameDimensionsList[0]);
                int frameCount = _image.GetFrameCount(dimension);
                _frameIndex = _frameIndex % frameCount;
                _image.SelectActiveFrame(dimension, _frameIndex);
                g.DrawImage(_image, (Width / 2) - (_image.Width / 2), (Height / 2) - (_image.Height / 2));
            }
        }

        private Size getStringSize(string text, System.Drawing.Font font, Graphics g)
        {
            Point startPoint = new Point(0, 0);
            TextFormatFlags flags = TextFormatFlags.NoPadding;
            Size proposedSize = new Size(int.MaxValue, int.MaxValue);
            Size size = TextRenderer.MeasureText(g, text, font, proposedSize, flags);
            return size;
        }
    }
}
